#include "MBox.h"


MBox::MBox(border_style border, BView *label, MView *content)
	: BBox(BRect(0,0,500,50),B_FOLLOW_NONE,
			B_WILL_DRAW|B_FRAME_EVENTS|B_NAVIGABLE_JUMP, border)
{
	labelview=label;
	contentview=content;

	if(label)
		SetLabel(label);
	if(content)
		AddChild(dynamic_cast<BView*>(content));
}

MBox::MBox(border_style border, char *label, MView *content)
	: BBox(BRect(0,0,500,50),B_FOLLOW_NONE,
			B_WILL_DRAW|B_FRAME_EVENTS|B_NAVIGABLE_JUMP, border)
{
	labelview=NULL;
	contentview=content;

	if(label)
		SetLabel(label);
	if(content)
		AddChild(dynamic_cast<BView*>(content));
}

MBox::~MBox()
{

}


void MBox::AddChild(BView *child)
{
	MView *mchild=dynamic_cast<MView*>(child);
	if(!mchild)
		debugger("MBox child must be an MView");
	contentview=mchild;
	BBox::AddChild(child);
}

const float contentinset=6.0;

minimax MBox::layoutprefs()
{
	mpm=ct_mpm;

	if(labelview)
	{
		MView *mview=dynamic_cast<MView*>(labelview);
		if(mview)
			lmpm=mview->layoutprefs();
		else
		{
			lmpm.mini.x=labelview->Bounds().Width()+1;
			lmpm.maxi.x=lmpm.mini.x;
			lmpm.mini.y=labelview->Bounds().Height()+1;
			lmpm.maxi.y=lmpm.mini.y;
		}
		labelheight=lmpm.mini.y;
	}
	else
	{
		BFont font; 
		GetFont(&font);
		font_height height;
		font.GetHeight(&height);
		labelheight=height.ascent+height.descent;
	}


	minimax cmpm;
	if(contentview)
		cmpm=contentview->layoutprefs();

	if(mpm.mini.x<cmpm.mini.x+contentinset*2)
		mpm.mini.x=cmpm.mini.x+contentinset*2;
	if(mpm.mini.y<cmpm.mini.y+labelheight+contentinset)
		mpm.mini.y=cmpm.mini.y+labelheight+contentinset;

	if(mpm.maxi.x<mpm.mini.x)
		mpm.maxi.x=mpm.mini.x;
	if(mpm.maxi.y<mpm.mini.y)
		mpm.maxi.y=mpm.mini.y;

	return mpm;
}




BRect MBox::layout(BRect rect)
{
	MoveTo(rect.LeftTop());
	ResizeTo(rect.Width(),rect.Height());

	if(contentview)
	{
		BRect childrect=rect;
		childrect.OffsetTo(0,0);
		childrect.top+=labelheight-contentinset;
		childrect.InsetBy(BPoint(contentinset,contentinset));
		contentview->layout(childrect);
	}
	if(labelview)
	{
		MView *mview=dynamic_cast<MView*>(labelview);
		if(mview)
		{
			BRect labelrect=labelview->Frame();
			labelrect.right=labelrect.left+lmpm.mini.x-1;
			labelrect.bottom=labelrect.top+lmpm.mini.y-1;
			mview->layout(labelrect);
		}
	}
	return rect;
}