/* TConfig.cpp
 *
 * read/write/default setting for pWarp/pWarpConfig
 */

#include "pwarp.h"
#include "TConfig.h"
#include <stdio.h>
#include <Rect.h>

TConfig::TConfig(bool find=true) {

	status = B_OK;

	/*
	 * Try opening the prefs
	 */
	prefs = new TPreferences("pWarp_prefs");
	if (prefs->InitCheck() != B_OK) {
		/*
		 * If not, set default values
		 */
		prefs->SetInt32("warp_treshold", DefaultTreshold());
		prefs->SetBool("enabled", DefaultEnabled());
		prefs->SetBool("popup_enabled", DefaultPopupEnabled());
		prefs->SetInt32("workspace_size", DefaultWorkspaceSize());
	} else {
		int32 version;
		if (prefs->FindInt32("version", &version) != B_OK) {
			version = T_CURRENT_VERSION - 1;
		}
		if (version < T_CURRENT_VERSION) {
			prefs->SetInt32("version", T_CURRENT_VERSION);

			// rewrite all attribs...
			prefs->SetInt32("warp_treshold", DefaultTreshold());
			prefs->SetBool("enabled", DefaultEnabled());
			prefs->SetBool("popup_enabled", DefaultPopupEnabled());
			prefs->SetInt32("workspace_size", DefaultWorkspaceSize());
		}
	}

	if (find) {
		// find the pWarp port...
		config_port = find_port("pwarp");
		if (config_port == B_NAME_NOT_FOUND) {
			status = !B_OK;
		}
	}
}

TConfig::~TConfig() {
	delete prefs;
}


//////////////////

status_t TConfig::InitCheck() {
	return status;
}

//////////////////

int32 TConfig::Treshold() {
	int32 time;
	if (prefs->FindInt32("warp_treshold", &time) != B_OK) {
		time = DefaultTreshold();
		SetTreshold(time);
	}
	return time;
}
int32 TConfig::DefaultTreshold() {
	return (int32)250;
}
void TConfig::SetTreshold(int32 time) {
	prefs->SetInt32("warp_treshold", time);

	int32 data=(int32)(time);
	write_port(config_port, T_CONFIG_CHANGE_TRESHOLD, (void*)&data, sizeof(int32) );
}

///////////////
	
bool TConfig::Enabled() {
	bool enabled;
	if (prefs->FindBool("enabled", &enabled) != B_OK) {
		enabled = DefaultEnabled();
		SetEnabled(enabled);
	}
	return enabled;
}

bool TConfig::DefaultEnabled() {
	return true;
}

void TConfig::SetEnabled(bool enabled) {
	prefs->SetBool("enabled", enabled);

	// update pWarp
	int data=0;
	if (enabled)
		write_port(config_port, T_CONFIG_ENABLE, (void*)&data, sizeof(int32) );
	else
		write_port(config_port, T_CONFIG_DISABLE, (void*)&data, sizeof(int32) );
}

///////////////
	
bool TConfig::PopupEnabled() {
	bool enabled;
	if (prefs->FindBool("popup_enabled", &enabled) != B_OK) {
		enabled = DefaultPopupEnabled();
		SetPopupEnabled(enabled);
	}
	return enabled;
}

bool TConfig::DefaultPopupEnabled() {
	return true;
}

void TConfig::SetPopupEnabled(bool enabled) {
	prefs->SetBool("popup_enabled", enabled);

	// update pWarp
	int data=0;
	if (enabled)
		write_port(config_port, T_CONFIG_POPUP_ENABLE, (void*)&data, sizeof(int32) );
	else
		write_port(config_port, T_CONFIG_POPUP_DISABLE, (void*)&data, sizeof(int32) );

}

///////////////
///////////////

BRect TConfig::Frame() {
	BRect rect;
	if (prefs->FindRect("window_frame", &rect) != B_OK) {
		rect = BRect( DefaultFrame() );
		SetFrame(rect);
	}
	return rect;
}

BRect TConfig::DefaultFrame() {
	return BRect(20,20,80,80);
}


void TConfig::SetFrame(BRect rect) {
	prefs->SetRect("window_frame", rect);
}

///////////////
///////////////

int32 TConfig::WorkspaceSize() {
	int32 size;
	if (prefs->FindInt32("workspace_size", &size) != B_OK) {
		size = DefaultWorkspaceSize();
		SetWorkspaceSize( size );
	}
	return size;
}

int32 TConfig::DefaultWorkspaceSize() {
	return 200;
}


void TConfig::SetWorkspaceSize(int32 size) {
	prefs->SetInt32("workspace_size", size);

	int32 data=(int32)(size);
	write_port(config_port, T_CONFIG_SET_WORKSPACE_SIZE, (void*)&data, sizeof(int32) );
}

// [Thursday 18 May 2000	17:44] Workspaces switcher a la X Windows. Shows the workspace window if moving the mouse to the position 0,0
// [Friday 19 May 2000	15:55] Version 0.2<BR>Added a Workspaces "auto-hide", configurable via pWarpConfig (activated by moving the mouse to 0,0).<BR>Fixed the preferences, so it would add the attribs not present to "pWarp_prefs".
