/* pWarpConfig.cpp
 *
 * Prefs panel for pWarp
 */

#include "pwarp.h"
#include "TConfig.h"

#include "layout/layout-all.h"
#include <Alert.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <MenuField.h>
#include <MenuBar.h>

#include "MBox.h"
#include <stdio.h>

#include <Rect.h>

#define T_ENABLE 			'enab'
#define T_POPUP_ENABLE 		'enpo'
#define T_BUTTON_REVERT 	'reve'
#define T_BUTTON_DEFAULTS 	'defa'

TConfig *g_Config;

class TWindow : public MWindow {
public:
	TWindow();
	virtual bool QuitRequested();
	virtual void MessageReceived(BMessage*);
	virtual void WorkspaceActivated(int32 workspace, bool active);
private:
	BRect rectify(BRect);
	bool m_Enabled;

	int m_CurrentWorkspace;

	BPopUpMenu *m_Workspaces;	

	MCheckBox *m_WarpingEnabled;
	MCheckBox *m_PopupEnabled;
	SpinButton *m_WarpingTreshold;
	SpinButton *m_PopupSize;
};


class TApplication : public MApplication {
public:
	TApplication();
	virtual void MessageReceived(BMessage*);
private:
	TWindow *m_Window;
};


TApplication::TApplication()
	: MApplication("application/x-vnd.tic-pwarpconfig") {

	if (g_Config->InitCheck() != B_OK) {
		(new BAlert("pWarpConfig", "pWarp not running!", "Ok"))->Go();
		be_app->PostMessage(B_QUIT_REQUESTED);
		return;
	}

	m_Window = new TWindow();
	m_Window->Show();
	Run();
}

void TApplication::MessageReceived(BMessage *message) {
	switch(message->what) {
		default: {
			MApplication::MessageReceived(message);
		} break;
	}
}

BRect TWindow::rectify(BRect rect) {
	return BRect(rect.left,rect.top,rect.left+1,rect.top+1);
}

void TWindow::WorkspaceActivated(int32 workspace, bool active) {
	printf("%d == %s\n", workspace, (active)?"true":"false");
}

TWindow::TWindow()
	: MWindow(rectify(g_Config->Frame()),"pWarpConfig",B_TITLED_WINDOW,B_ASYNCHRONOUS_CONTROLS) {

	//SetFlags(B_NOT_RESIZABLE);
	SetFlags(B_NOT_V_RESIZABLE);
	SetWorkspaces(B_CURRENT_WORKSPACE);
	
	//********************
	m_WarpingEnabled = new MCheckBox("Enabled", new BMessage(T_ENABLE) );
	m_WarpingEnabled->SetValue( g_Config->Enabled()?1:0 );
	
	//********************
	m_PopupEnabled = new MCheckBox("Enabled", new BMessage(T_POPUP_ENABLE) );
	m_PopupEnabled->SetValue( g_Config->PopupEnabled()?1:0 );
	
	//************************
	m_WarpingTreshold = new SpinButton("Treshold", SPIN_INTEGER);
	m_WarpingTreshold->SetMaximum(1000);
	m_WarpingTreshold->SetValue((long int)g_Config->Treshold() );
	m_WarpingTreshold->SetStepSize(10.0);

	//************************
	m_PopupSize= new SpinButton("Size", SPIN_INTEGER);
	m_PopupSize->SetMaximum(400.0);
	m_PopupSize->SetValue((long int)g_Config->WorkspaceSize() );
	m_PopupSize->SetStepSize(5.0);

	//************************
	m_Workspaces = new BPopUpMenu("Current Workspace");
	m_Workspaces->AddItem( new BMenuItem( "Current Workspace", new BMessage('curn')));
	m_Workspaces->AddItem( new BMenuItem( "All Workspaces", new BMessage('alwk')));
	
	BMenuField *m_WorkspaceField = new BMenuField(BRect(10,10,10,10), "moo", NULL, m_Workspaces);
	m_WorkspaceField->SetDivider(0.0f);
	
	DivideSame(m_WarpingTreshold,m_PopupSize,0);

	MView *topview =
			new HGroup(
				new Space(minimax(200,200,200,200)), // new Space(), put the BView here instead.
				new VGroup(
					new Space(minimax(10,10,10,10)),
					new MBox(B_FANCY_BORDER, m_WorkspaceField,
						new HGroup(
							new Space(minimax(5,5,5,5)),
							new VGroup (
								new Space(minimax(5,5,5,5)),
								new MBox (B_PLAIN_BORDER,"Warping",
									new VGroup (
										m_WarpingEnabled,
										m_WarpingTreshold,
										0
									)
								),
								new Space(minimax(5,5,5,5)),
								new MBox(B_PLAIN_BORDER,"Workspaces",
									new VGroup (
										m_PopupEnabled,
										m_PopupSize,
										0
									)
								),
								new Space(minimax(5,5,5,5)),
								0
							),
							new Space(minimax(5,5,5,5)),
							0
						)
					),
					new Space(minimax(10,10,10,10)),
					0
				),
				new Space(minimax(10,10,10,10)),
				0
			);

	AddChild(dynamic_cast<BView*>(topview));
}

bool TWindow::QuitRequested() {

	g_Config->SetFrame(this->Frame());

	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

void TWindow::MessageReceived(BMessage *message) {
	switch(message->what) {

		case '!spn': {

			// one of the spinbuttons. look in the 'source' field for which
			
			BControl *spin;
			message->FindPointer("source", (void**)&spin);

			if ((SpinButton*)spin == m_PopupSize) {
				g_Config->SetWorkspaceSize((int32)m_PopupSize->Value());
			} else if ((SpinButton*)spin == m_WarpingTreshold) {
				g_Config->SetTreshold(m_WarpingTreshold->Value());
			}
			
		} break;

		case T_ENABLE: {
			g_Config->SetEnabled( !g_Config->Enabled() );
		} break;
		
		case T_POPUP_ENABLE: {
			g_Config->SetPopupEnabled( !g_Config->PopupEnabled() );
		} break;

		default:
			message->PrintToStream();
			MWindow::MessageReceived(message);
			break;
	}
}


int main() {

	g_Config = new TConfig();
	new TApplication();
	delete g_Config;
	
	return 0;
}
// [Thursday 18 May 2000	17:44] Workspaces switcher a la X Windows. Shows the workspace window if moving the mouse to the position 0,0
// [Friday 19 May 2000	15:55] Version 0.2<BR>Added a Workspaces "auto-hide", configurable via pWarpConfig (activated by moving the mouse to 0,0).<BR>Fixed the preferences, so it would add the attribs not present to "pWarp_prefs".
