/* pwarp_filter.cpp
 *
 * pwarp_filter main file
 *
 * (c) 1999-2000 by Mikael Jansson
*/

#include "pwarp.h"
#include "TConfig.h"
#include <InterfaceDefs.h>
#include <Message.h>
#include <Screen.h>
#include <string.h>
#include <stdio.h>
#include <Messenger.h>
#include <OS.h>
#include <Window.h>
#include <Roster.h>

static int32 animator_thread_func( void *data ) {

	const char *app_sig = "application/x-vnd.Be-WORK";

	while( true ) {

		workspace_package_t *wp = (workspace_package_t*)data;

		// if not running Workspaces, start it.
		app_info app;		
		be_roster->GetAppInfo(app_sig, &app);
		if (app.team == -1) {
			// not good at all... sigh.
			be_roster->Launch(app_sig); // beware, crashes system!
		}

		BMessenger msger(app_sig);
		BMessage looks(B_SET_PROPERTY),reply;
		looks.AddSpecifier("Flags");
		looks.AddSpecifier("Window", (long int)0);
		int32 the_data = B_NOT_MOVABLE|B_NOT_RESIZABLE|B_WILL_ACCEPT_FIRST_CLICK;
		looks.AddInt32("data", the_data);
		msger.SendMessage(&looks,&reply);

		looks.MakeEmpty();
		looks.AddSpecifier("Look");
		looks.AddSpecifier("Window", (long int)0);
		the_data = B_MODAL_WINDOW_LOOK;
		looks.AddInt32("data", the_data);
		msger.SendMessage(&looks,&reply);

		// show the popup
		if (wp->hiding == false) {
			
			// hide.
			BMessage msg_hidden(B_SET_PROPERTY);
			msg_hidden.AddSpecifier("Hidden");
			msg_hidden.AddSpecifier("Window", (long int)0);
			msg_hidden.AddBool("data", true);
			msger.SendMessage(&msg_hidden,&reply);
		
			BRect position(wp->workspaces_rect);
			position.OffsetBy(0,-(wp->workspaces_rect.bottom - wp->workspaces_rect.top));
		
			// move it into place.
			BMessage msg(B_SET_PROPERTY);
			msg.AddSpecifier("Frame");
			msg.AddSpecifier("Window", (long int)0);	
			msg.AddRect("data", position);
			msger.SendMessage(&msg,&reply);
		
			// show.
			msg_hidden.ReplaceBool("data", false);
			msger.SendMessage(&msg_hidden,&reply);
			
			// animate.
			for (int i=0; i<(int)wp->workspaces_rect.bottom+1; i+=2) {
				msg.ReplaceRect("data", position);
				msger.SendMessage(&msg,&reply);
				
				position.OffsetBy(0,2);
			}
		
		 } else {

			BRect position(wp->workspaces_rect);
			BMessage msg(B_SET_PROPERTY),reply;
			msg.AddSpecifier("Frame");
			msg.AddSpecifier("Window", (long int)0);	
			msg.AddRect("data", position);

			// animate.
			for (int i=0; i<(int)wp->workspaces_rect.bottom+1; i+=2) {
				msg.ReplaceRect("data", position);
				msger.SendMessage(&msg,&reply);
				position.OffsetBy(0,-2);
			}
	
			// show.
			msg.MakeEmpty();
			msg.AddSpecifier("Hidden");
			msg.AddSpecifier("Window", (long int)0);
			msg.AddBool("data", true);
			msger.SendMessage(&msg,&reply);
		 }

		wp->animating=false;		
		suspend_thread(find_thread(NULL));

	} // end endless loop
}

//###########################

TInputServerFilter::TInputServerFilter() {
	
	/**
	 * Resetting/Initializing
	 */
	for (int i=0;i<T_MAX_EVENTS;i++) {
		events[i].x=events[i].y=1;
		events[i].when=system_time();
	}
	
	dynamic_num_workspaces = 255;
	workspaces = NULL;

	soft_warping_time = event_index = 0;;
	config_port = -1;
	
	number_events_in_list=0;
	event_index=0;

	is_mouse_down = false;

	
	/* Start the communication
	 * between pWarp and the prefs panel.
	 */
	config_port = create_port(30, "pwarp");
	
	/*
	 * Configuration goes here
	*/
	TConfig config(false);
	config_warp_treshold = (bigtime_t)(1000*config.Treshold());
	is_enabled = config.Enabled();
	is_popup_enabled = config.PopupEnabled();

	/* workspaces animation and stuff */
	is_inside_workspaces = false;
	int size = config.WorkspaceSize();
	workspace_package.workspaces_rect.Set(0,0, size,size);
	workspace_package.animating=false;

	animator_thread = spawn_thread(animator_thread_func, "Mr. Animator", B_NORMAL_PRIORITY, (void *)&workspace_package);
	suspend_thread(animator_thread);

}

TInputServerFilter::~TInputServerFilter() {
	if (workspaces != NULL) delete workspaces;
	if (config_port != B_BAD_VALUE && config_port != B_NO_MORE_PORTS && config_port != -1) {
		delete_port(config_port);
	}
}


/* Check for input server msg's.
 * Working!
 */
filter_result TInputServerFilter::Filter(BMessage *msg, BList *)
{

	/* here's a good place to check if the configuration has changed */
	while( port_count(config_port) != 0 ) {
		
		int32 what,data;
		read_port(config_port, &what, &data, sizeof(int32));

		/* check what the user wants */
		switch( what ) {
			case T_CONFIG_CHANGE_TRESHOLD: {
				config_warp_treshold = (bigtime_t)(data * 1000);
			} break;
			
			case T_CONFIG_ENABLE: {
				is_enabled = true;
			} break;

			case T_CONFIG_DISABLE: {
				is_enabled = false;
			} break;

			case T_CONFIG_POPUP_ENABLE: {
				is_popup_enabled = true;
			} break;

			case T_CONFIG_POPUP_DISABLE: {
				is_popup_enabled = false;
			} break;
			
			case T_CONFIG_SET_WORKSPACE_SIZE: {
				workspace_package.workspaces_rect.Set(0,0,(float)data,(float)data);
			} break;
		}
	}
	

	filter_result res = B_DISPATCH_MESSAGE;

	if (!workspace_package.animating) {
		switch( msg->what ) {
	
			case B_MOUSE_DOWN: {
				is_mouse_down = true;
			} break;
			
			case B_MOUSE_UP: {
				is_mouse_down = false;
			}
			
			case B_MOUSE_MOVED: {

				BPoint where;
				msg->FindPoint("where", &where);

				if (is_popup_enabled) {
					if (is_inside_workspaces) {
						if (where.x > workspace_package.workspaces_rect.right+10 || where.y > workspace_package.workspaces_rect.bottom+10) {
							is_inside_workspaces = false;
							hide_workspaces();
						}
					}
				}

				// special case, if we want to show Workspaces
				if (where.x == 0 && where.y == 0) {
					if (is_popup_enabled) {
						show_workspaces();
					}
				} else {
					if (is_enabled) {
						add_event(msg);
		
						int i_warp = is_warping();
						if (i_warp != T_NOT_WARPING) {
							warp(i_warp);
						}
					}
				}
			} break;
				
			default: {
			} break;
			
		}
	}
			
	return res;
}


void TInputServerFilter::show_workspaces() {					

	if (is_inside_workspaces)
		return;
	else
		is_inside_workspaces=true;

	// run a thread here that does the actual animation...
	workspace_package.animating=true;
	workspace_package.hiding=false;
	resume_thread(animator_thread);

}

void TInputServerFilter::hide_workspaces() {					

	// run a thread here that does the actual animation...
	workspace_package.animating=true;
	workspace_package.hiding=true;
	resume_thread(animator_thread);

}


/* add an event to the event FILO
 * Working!
 */
inline void TInputServerFilter::add_event(BMessage *msg) {
	
	BPoint where;
	bigtime_t when;
	msg->FindPoint("where", &where);
	msg->FindInt64("when",  &when );
	
	/* save the old event */
	memcpy(&previous_event, &current_event, sizeof(event_t));

	/* set the current event */
	current_event.when = when;
	current_event.x = (int)where.x;
	current_event.y = (int)where.y;

	/* another is neccessary when resetting the event list in Hard Warp mode */
	number_events_in_list++;
	if (number_events_in_list>=T_MAX_EVENTS)
		number_events_in_list=T_MAX_EVENTS;

	/* FiLO buffer */
	event_index++;
	if (event_index >= number_events_in_list) {
		event_index = 0;
	}
	
	events[event_index].when = current_event.when;
	events[event_index].x = current_event.x;
	events[event_index].y = current_event.y;
	
	/* set the event that's going to be kicked out
	 * the next event
	 */
	int old;
	if (event_index+1 >= number_events_in_list)
		old = 0;
	else
		old = event_index+1;
	
	old_event.when = events[old].when;
	old_event.x = events[old].x;
	old_event.y = events[old].y;
}

int TInputServerFilter::is_warping() {

	int which=T_NOT_WARPING;
	
	/* have we hit the border? if not, bail out */

	screen_rect = BScreen(B_MAIN_SCREEN_ID).Frame();

	if ( pos_x() == (int)screen_rect.left) {
		which=T_WARPING_LEFT;
	}
	else if (pos_x() == (int)screen_rect.right) {
		which=T_WARPING_RIGHT;
	}
	else if (pos_y() == (int)screen_rect.top) {
		which=T_WARPING_UP;
	}
	else if (pos_y() == (int)screen_rect.bottom) {
		which=T_WARPING_DOWN;
	}

	/* ... continue */
	if (which != T_NOT_WARPING) {

		/* bail out if mouse button pressed */
		if (is_mouse_down == false) {
			// determine the time spent at the border of the screen....
			switch( which ) {
				case T_WARPING_UP:
				case T_WARPING_DOWN: {

					/* set prev_event and curr_event to be event_t's instead */
					if (previous_event.y == current_event.y) {
						soft_warping_time += (current_event.when - previous_event.when);
					} else {
						soft_warping_time = 0;
					}
				} break;	

				case T_WARPING_RIGHT:
				case T_WARPING_LEFT: {
					if (previous_event.x == current_event.x) {
						soft_warping_time += (current_event.when - previous_event.when);
					} else {
						soft_warping_time = 0;
					}
				} break;
			}

			if (soft_warping_time < config_warp_treshold) {
				// nope, not this time.
				which=T_NOT_WARPING;
			} else {
				// warp where you want to, ie, _don't_ set which to T_NOT_WARPING
				soft_warping_time = 0;
			}
		} else {
			which = T_NOT_WARPING;
		}
	} // end if T_NOT_WARPING

	return which;
}


/* Calculate the layout of the Workspaces prefs.
 * Working!
 */
void TInputServerFilter::recalculate_layout() {

	int i,j;
	int x,y;
	int best_i,best_j;
	int best_delta=255;
		
	for (i=0; i<dynamic_num_workspaces; i++) {
		for (j=0; j<dynamic_num_workspaces; j++) {
		
			if (i*j == dynamic_num_workspaces) {
					
				int delta=i-j;
				if (delta<0) delta *= (-1);
				
				if (best_delta>delta) {
					best_delta = delta;
					best_i = i;
					best_j = j;	
				}
			}
		}
	}
	
	set_width( best_i );
	set_height( best_j );
	
	/* now, calculate the actual grid */
	
	if (workspaces != NULL) {
		delete workspaces;
	}
	workspaces = new workspaces_t[get_width()*get_height()];

	int index=0;
	for (y=0; y<get_height(); y++) {
		for (x=0; x<get_width(); x++, index++) {

			/* warp wherever you want to .. */
			
			workspaces[index].left = true;
			workspaces[index].right = true;
			workspaces[index].up = true;
			workspaces[index].down = true;

			/* mark the directions not possible */
			
			if (x==0) {
				workspaces[index].left = false;
			} else if (x==(get_width()-1)) {
				workspaces[index].right = false;
			}
			if (y==0) {
				workspaces[index].up = false;
			} else if (y==(get_height())-1) {
				workspaces[index].down = false;
			}
		}
	}
}

/* Do the warp.
 * Working!
 */
void TInputServerFilter::warp(int direction) {

	if (dynamic_num_workspaces != count_workspaces()) {
		dynamic_num_workspaces = count_workspaces();

		recalculate_layout();
	}		

	int new_workspace;
	switch( direction ) {
	
		case T_WARPING_UP:
			new_workspace = boundaries(current_workspace()-get_width());
			if (new_workspace != (-1))
				if (workspaces[current_workspace()].up==false)
					new_workspace = (-1);
			break;
		
		case T_WARPING_DOWN:
			new_workspace = boundaries(current_workspace()+get_width());
			if (new_workspace != (-1))
				if (workspaces[current_workspace()].down==false)
					new_workspace = (-1);
			break;
	
		
		case T_WARPING_LEFT:
			new_workspace = boundaries(current_workspace()-1);
			if (new_workspace != (-1)) {
				if (workspaces[current_workspace()].left==false) {
					new_workspace = (-1);
				}
			}

			break;
		
		case T_WARPING_RIGHT:
			new_workspace = boundaries(current_workspace()+1);
			if (new_workspace != (-1))
				if (workspaces[current_workspace()].right==false)
					new_workspace = (-1);

			break;		
	
		default: new_workspace = -1; break;
	}
	
	if (new_workspace != (-1) ) {
		activate_workspace( new_workspace );
	}
}

//##############################

BInputServerFilter* instantiate_input_filter()
{
	return (new TInputServerFilter());
}
// [Thursday 18 May 2000	17:44] Workspaces switcher a la X Windows. Shows the workspace window if moving the mouse to the position 0,0
// [Friday 19 May 2000	15:55] Version 0.2<BR>Added a Workspaces "auto-hide", configurable via pWarpConfig (activated by moving the mouse to 0,0).<BR>Fixed the preferences, so it would add the attribs not present to "pWarp_prefs".
