/* pwarp_filter.h
 *
 * pwarp_filter header file
 *
 * (c) 1999-2000 by Mikael Jansson
*/

#ifndef __PWARP_FILTER_H
#define __PWARP_FILTER_H

#include <Rect.h>
#include <add-ons/input_server/InputServerFilter.h>
#include "TPreferences.h"

#define T_PWARP_PORT					"pwarp_config_port"
#define T_CONFIG_CHANGE_TRESHOLD 		'ccth'
#define T_CONFIG_DISABLE		 		'cddi'
#define T_CONFIG_ENABLE		 			'cden'
#define T_CONFIG_POPUP_DISABLE			'podi'
#define T_CONFIG_POPUP_ENABLE 			'poen'
#define T_CONFIG_SET_WORKSPACE_SIZE		'size'

#define T_PWARP_PREFERENCES 			"pWarp_prefs"

#define T_MAX_EVENTS 	30 // in practice this is the number of pixels

#define T_SOFT_WARP		0
#define T_HARD_WARP		1

#define T_NOT_WARPING		0
#define T_WARPING_RIGHT		1
#define T_WARPING_LEFT		2
#define T_WARPING_UP		3
#define T_WARPING_DOWN		4

#define T_HORIZONTAL_WARP	1
#define T_VERTICAL_WARP		2

typedef struct workspace_package_typ {
	bool animating; // controlled by thread
	bool hiding; // hiding or showing..		
	BRect workspaces_rect;
	BRect temp_rect; // the one that's animating, for easy access from the outside.
	bool cursor_inside;
} workspace_package_t;


class TInputServerFilter : public BInputServerFilter {

	public:
		TInputServerFilter();
		~TInputServerFilter();
		
		virtual filter_result Filter(BMessage*, BList*);

	private:

		/* check with the event history
		 * if we should warp yet */
		int is_warping();

		/* warp to the workspace the
		 * user is pointing his device at */
		void warp(int direction);
		
		/* add a mouse event to the global FiLO queue */
		void add_event( BMessage *msg );
		
		/* get the position ... */
		int pos_x() const { return current_event.x; }
		int pos_y() const { return current_event.y; }

		/* set the dimension */
		int set_width(int w) {dynamic_x_size=w;}
		int set_height(int h) {dynamic_y_size=h;}
		
		int get_width() const { return dynamic_x_size; }
		int get_height() const { return dynamic_y_size; }

		/* check boundarier */
		int boundaries(int n) {
			if (n<0 || n>=dynamic_num_workspaces)
				n = (-1);
			return n;
		}
	
		/* recalculate the row/cell layout */
		void recalculate_layout();
		
		/* invalidated every time a message is passed */
		BRect screen_rect;
		
		struct event_t {
			bigtime_t when;
			int32 x,y;
		};
		event_t events[T_MAX_EVENTS];
		event_t current_event;
		event_t previous_event;
		event_t old_event;
		int event_index;
		int number_events_in_list;

		/* for communication with the prefs panel */
		port_id config_port;

		/* MouseDown's */
		bool is_mouse_down;

		/* Are we currently showing Workspaces? */
		bool is_inside_workspaces;
		
		/* config: enabled? */
		bool is_enabled;
		bool is_popup_enabled;
		
		/* the size of the workspaces windov */

		workspace_package_t workspace_package;
		void show_workspaces();
		void hide_workspaces();
		thread_id animator_thread;

		struct workspaces_t {
			bool left,right,up,down;
		};
		workspaces_t *workspaces;

		uint8 dynamic_num_workspaces;
		uint8 dynamic_x_size;
		uint8 dynamic_y_size;
		
		/* the time spent at one of the screen's borders */
		bigtime_t soft_warping_time;

		/* time to hit the border in HARD_MODE,
		 * and time near the edge in SOFT_MODE */
		bigtime_t config_warp_treshold; 
};


extern "C" _EXPORT BInputServerFilter* instantiate_input_filter();

#endif // __PWARP_FILTER_H